# SPDX-FileCopyrightText: 2024 Deutsche Klimarechenzentrum
#
# SPDX-License-Identifier: CC-BY-4.0

import argparse
import json
import os
import shutil
import subprocess
import tempfile

from git import GitCommandError, Repo
from rich.console import Console
from rich.progress import Progress, SpinnerColumn, TextColumn
from rich.prompt import Prompt


class FrevaPluginTemplateCLI:
    def __init__(self):
        self.console = Console()
    def clone_repo(self, repo_url, clone_dir, branch=None):
        """
        This function clones a Git repository from a given URL to a specified directory.
        Optionally, it checks out a specific branch.
        """
        console = Console()
        try:
            if branch:
                Repo.clone_from(repo_url, clone_dir, branch=branch)
            else:
                Repo.clone_from(repo_url, clone_dir)
            return True
        except GitCommandError as e:
            console.print(f"[red]Error: Failed to clone repository {repo_url}[/red]\n{e}")
            return False

    def get_input(self, index, valid_options, default_value, variable_name):
        """
        Prompts the user for input, validates it against a set of valid options, and returns the selected option.

        Args:
            index (str): The index to display the progress.
            valid_options (list): A list of valid options.
            default_value (str): The default value to use if no input is provided.
            variable_name (str): The name of the variable being set.

        Returns:
            str: The validated user input or the default value.
        """
        valid_options_lower = [option.lower() for option in valid_options]
        if isinstance(valid_options, list):
            valid_options = ", ".join(valid_options)
        else:
            valid_options = str(valid_options)
        prompt_message = f"  {index} {variable_name} [cyan][bold]({valid_options})[/bold][/cyan]  --default: {default_value} "
        while True:
            user_input = Prompt.ask(prompt_message).strip().lower()
            if user_input == "":
                self.console.print(f"         [yellow]No input provided. Defaulting to '{default_value}'[/yellow]")
                return default_value.lower()
            elif user_input in valid_options_lower:
                return user_input
            else:
                self.console.print(f"          [red]Invalid input. Please enter one of the following: [bold]{valid_options}[/bold][/red]")

    def create_template(self):
        # Prompt the user for inputs
        plugin_visibility = self.get_input("[grey53][-3/*][/grey53]", ["public", "private"], "public","plugin_visibility")
        use_documentation = self.get_input("[grey53][-2/*][/grey53]", ["yes", "no"], "yes","use_documentation")
        use_tests = self.get_input("[grey53][-1/*][/grey53]", ["yes", "no"], "yes", "use_tests")
        use_submodules = self.get_input("[grey53][0/*][/grey53]", ["yes", "no"], "no", "use_submodules")

        context = {
            "plugin_authors": "Firstname Lastname, Another-Firstname Lastname",
            "plugin_author_emails": "someone@dkrz.de, someoneelse@dkrz.de",
            "plugin_maintainers": "{{ cookiecutter.plugin_authors }}",
            "plugin_maintainer_emails": "{{ cookiecutter.plugin_author_emails }}",
            "project_id": "ch1187",
            "gitlab_host": "gitlab.dkrz.de",
            "gitlab_username": "ch1187/plugins4freva",
            "git_remote_protocoll": ["https", "ssh"],
            "institution": "Deutsche Klimarechenzentrum",
            "institution_url": "https://www.dkrz.de",
            "conda_channels": "conda-forge",
            "conda_dependencies": "r-base, pip",
            "add_assets": ["yes", "no"],
            "pip_dependencies": "numpy",
            "plugin_title": "My Freva Plugin",
            "plugin_slug": "{{ cookiecutter.plugin_title.lower() | slugify }}",
            "plugin_folder": "{{ cookiecutter.plugin_slug.lower().replace('-', '_') }}",
            "plugin_short_description": "{{ cookiecutter.plugin_title }}",
            "keywords": "",
            "_extensions": ["local_extensions.UnderlinedExtension"]
        }

        if plugin_visibility == "public":
            context["copyright_holder"] = "{% if cookiecutter.institution == 'Deutsche Klimarechenzentrum' %}Deutsche Klimarechenzentrum{% else %}{{ cookiecutter.institution }}{% endif %}"
            context["copyright_year"] = "{% now 'utc', '%Y' %}"
            context["code_license"] = "EUPL-1.2"
            context["use_reuse"] = ["yes", "no"]
        elif plugin_visibility == "private":
            context["code_license"] = "proprietary"

        if use_documentation == "yes":
            context["documentation_license"] = "CC-BY-4.0"
            context["documentation_url"] = "https://{{ cookiecutter.plugin_slug }}.readthedocs.io/en/latest/"
            context["use_markdown_for_documentation"] = ["yes", "no"]

        if use_tests == "yes":
            context["supplementary_files_license"] = "CC0-1.0"
            context["use_dkrz_runner"] = ["yes", "no"]
        if use_submodules == "yes":
            context["use_submodules"] = "yes"
            context["submodule_repos"] = "https://gitlab.dkrz.de/freva/plugins4freva/mapfactory.git"
        else:
            context["use_submodules"] = "no"
        with tempfile.TemporaryDirectory() as tempdir:
            template_dir = os.path.join(tempdir, 'template')
            repo_url = "https://gitlab.dkrz.de/ch1187/plugins4freva/freva-plugin-template.git"

            if os.path.exists('.git'):
                with Progress(SpinnerColumn(), TextColumn("[progress.description]{task.description}"), transient=True) as progress:
                    task = progress.add_task(description="Creating template configuration file...", total=None)
                    shutil.copytree('.', template_dir)
                    progress.update(task, completed=True)
                    progress.refresh()
            else:
                if not self.clone_repo(repo_url, template_dir, "main"):
                    raise SystemExit(1)

            cookiecutter_json_path = os.path.join(template_dir, 'cookiecutter.json')
            with open(cookiecutter_json_path, 'w') as json_file:
                json.dump(context, json_file, indent=4)

            cmd = f'cruft create {repo_url} --skip .git --skip .mypy_cache --directory {template_dir}'
            print("  ⏳ Processing Cruft...", end='\r')
            os.system(cmd)
            print(" " * 50, end='\r')
    def update_template(self):
        """
        Updates the Freva Plugin template to the latest version.
        Optionally creates a new Git branch for the update and provides instructions for reviewing and committing the changes.
        """
        create_branch = self.get_input("Do you want to create a new branch for the update?", ["yes", "no"], "yes", "")

        if create_branch == "yes":
            branch_name = self.get_input("Enter the branch name:", "letters (a-z, A-Z), numbers (0-9), -, _, ., /.", "update-skeleton", "")
            Prompt.ask("  Enter the branch name (e.g., update-skeleton):").strip()
            subprocess.run(['git', 'branch', branch_name])
            subprocess.run(['git', 'checkout', branch_name])
            subprocess.run(['cruft', 'update'])

            self.console.print("""
                [green]Next steps:
                1. Review the changes made by 'cruft update'.
                2. Make sure everything is working by executing the test suite:
                    git add .
                    pre-commit run
                    # Resolve any issues pre-commit may bring up and once it is working, run the test suite
                    make dev-install
                    tox

                3. If everything is working, commit the changes:
                    git commit -m "Updated package skeleton"

                4. Push the changes to the remote repository:
                    git push origin <branch_name>

                5. Create a merge request.[/green]
            """)
        else:
           self.console.print(
                "[red]⚠️ It is unsafe to make changes directly in the main branch. It is strongly recommended to create a new branch, review and then merge to avoid any conflicts. If you still wish to work in the main branch, please ensure all your changes are committed before proceeding. To continue in the main branch, run [yellow][bold]cruft update[/bold][/yellow] directly.[/red]"
            )


    def run(self):
        """
        Main entry point for the Freva Plugin Template CLI.
        """
        parser = argparse.ArgumentParser(description="Freva Plugin Template CLI")
        subparsers = parser.add_subparsers(dest='command', help='Sub-command help')

        # Create subparser for the 'create' command
        create_parser = subparsers.add_parser('create', help='Create a new Freva plugin template')
        create_parser.set_defaults(func=self.create_template)

        # Create subparser for the 'update' command
        update_parser = subparsers.add_parser('update', help='Update an existing Freva plugin template')
        update_parser.set_defaults(func=self.update_template)

        args = parser.parse_args()

        if hasattr(args, 'func'):
            args.func()
        else:
            parser.print_help()


def main():
    cli = FrevaPluginTemplateCLI()
    cli.run()

if __name__ == "__main__":
    cli = FrevaPluginTemplateCLI()
    cli.run()
