# SPDX-FileCopyrightText: 2023-2024 Helmholtz-Zentrum hereon GmbH, Deutsches Klimarechenzentrum GmbH
#
# SPDX-License-Identifier: MIT

import os
import shutil
import sys
import git
import subprocess as spr
import glob
import requests
from urllib.parse import urlparse
from time import sleep
import json

def get_current_branch_name():
    return spr.check_output(['git', 'branch', '--show-current']).decode('utf-8').strip()

{% if 'use_markdown_for_documentation' in cookiecutter and cookiecutter.use_markdown_for_documentation == "yes" %}
for fname in glob.glob("docs/*.rst"):
    os.remove(fname)
{% else %}
for fname in glob.glob("docs/*.md"):
    os.remove(fname)
{% endif %}

{% if 'documentation_license' not in cookiecutter %}
import shutil
shutil.rmtree("docs")
{% endif %}

# make assets folder if add_assets is set to yes
{% if 'add_assets' in cookiecutter and cookiecutter.add_assets == "yes" %}
os.makedirs("assets", exist_ok=True)
{% endif %}

{% if 'use_submodules' in cookiecutter and cookiecutter.use_submodules == "yes" %}
# add submodule
{%- for submodule_repo in cookiecutter.submodule_repos.split(',') %}
repo = git.Repo.init(".")
repo.create_submodule("src/{{ get_repo_name(submodule_repo) }}", "src/{{ get_repo_name(submodule_repo) }}", url="{{ submodule_repo }}")
{%- endfor %}
{% endif %}
# -----------------------------------------------------------------------------
# ------------ read environment variables -------------------------------------
# -----------------------------------------------------------------------------

# check if we should be silent
if os.getenv("SILENT_HOOKS"):
    silent = True
    spr_kwargs = dict(stdout=spr.DEVNULL, stderr=spr.STDOUT)
else:
    silent = False
    spr_kwargs = {}

# check if the plugin is initialized in an already existing git repo
if os.getenv("PARENT_GIT_REPO"):
    parent = os.getenv("PARENT_GIT_REPO")
else:
    parent = None

# check if we should run pre-commit hooks in the beginning
skip_pre_commit = parent is not None or os.getenv("SKIP_PRE_COMMIT")

def run_command(command, spr_kwargs):
    try:
        spr.check_call(command, **spr_kwargs)
        sleep(5)
    except spr.CalledProcessError as e:
        print(f"Command {command} failed with error: {e}")
        sys.exit(e.returncode)


{% if 'use_reuse' in cookiecutter and cookiecutter.use_reuse == "yes" %}
spr.check_call(
    [
        sys.executable,
        ".reuse/add_license.py",
        "code",
        "--recursive",
        "README.md",
    ],
    **spr_kwargs,
)
{% if 'documentation_license' in cookiecutter %}
spr.check_call(
    [
        sys.executable,
        ".reuse/add_license.py",
        "docs",
        "--recursive",
        "docs/Makefile",
        "docs/make.bat",
        "docs/requirements.txt",
        "docs/conf.py",
    ] + glob.glob("docs/*.rst") + glob.glob("docs/*.md") + glob.glob("docs/_static/*") + glob.glob("docs/_templates/*"),
    **spr_kwargs,
)
{% endif %}
{% if 'supplementary_files_license' in cookiecutter %}
spr.check_call(
    [
        sys.executable,
        ".reuse/add_license.py",
        "supp",
        "--recursive",
        ".gitignore",
        ".pre-commit-config.yaml",
        "pyproject.toml",
        ".gitattributes",
        ".gitlab-ci.yml",
        ".readthedocs.yaml",
        ".vscode/settings.json",
        "CHANGELOG.md",
        "MANIFEST.in",
        "Makefile",
        "tox.ini",
    ],
    **spr_kwargs,
)
{% endif %}

{% if 'use_submodules' in cookiecutter and cookiecutter.use_submodules == "yes" %}
spr.check_call(
    [
        sys.executable,
        ".reuse/add_license.py",
        "supp",
        "--recursive",
        ".gitmodules",
    ],
    **spr_kwargs,
)
{% endif %}
spr.check_call(
    [
        sys.executable,
        ".reuse/add_license.py",
        "supp",
        "--skip-unrecognised",
        "--style",
        "python",
        ".flake8",
        "CITATION.cff",
        ".reuse/shortcuts.yaml",
    ],
    **spr_kwargs,
)
spr.check_call(
    [
        sys.executable,
        ".reuse/add_license.py",
        "code",
        "--recursive",
        "src/{{ cookiecutter.plugin_folder }}",
        "{{cookiecutter.plugin_slug}}-env.yml",
        "{{cookiecutter.plugin_slug}}-wrapper-api.py",
        "src/tests/conftest.py",
        "src/tests",
        ".reuse/add_license.py",
    ],
    **spr_kwargs,
)
os.makedirs("docs/_static", exist_ok=True)
{% if 'documentation_license' in cookiecutter %}
if not silent:
    print("Download license image")
response = requests.get(
    "http://mirrors.creativecommons.org/presskit/buttons/88x31/png/{{ cookiecutter.documentation_license.split('-', 1)[1].rsplit('-', 1)[0].lower() }}.png"
)

with open("docs/_static/license_logo.png", "wb") as f:
    f.write(response.content)
spr.check_call(
    [
        "reuse",
        "--root",
        ".",
        "annotate",
        "--year",
        "{% now 'utc', '%Y' %}",
        "--license",
        "CC-BY-4.0",
        "--copyright",
        "Creative Commons",
        "docs/_static/license_logo.png",
    ],
    **spr_kwargs,
)
{% endif %}
spr.check_call(["reuse","--root",".", "download", "--all"], **spr_kwargs)

{% endif %}

if not silent:
    print("Fixing files for plugin at {{cookiecutter.plugin_slug}}")

if parent:
    repo = git.Repo(parent)
else:
    repo = git.Repo.init(".", mkdir=False)

    {% if 'git_remote_protocoll' in cookiecutter and cookiecutter.git_remote_protocoll == "ssh" %}
    repo.create_remote(
        "origin",
        "git@{{ cookiecutter.gitlab_host }}:{{ cookiecutter.gitlab_username }}/{{ cookiecutter.plugin_slug }}.git",
    )
    {% else %}
    repo.create_remote(
        "origin",
        "https://{{ cookiecutter.gitlab_host }}/{{ cookiecutter.gitlab_username }}/{{ cookiecutter.plugin_slug }}.git",
    )
    {% endif %}

repo.git.add(".")



default_branch = get_current_branch_name()

spr.check_call(["pre-commit", "install"], **spr_kwargs)
# make a silent run first to fix the files
if not skip_pre_commit:
    spr.call(["pre-commit", "run"], stdout=spr.DEVNULL)

# add the files again after pre-commit hook has been run
repo.git.add(".")

if not skip_pre_commit:
    spr.check_call(["pre-commit", "run"], **spr_kwargs)

for dirpath, dirnames, filenames in os.walk("."):
    if "__pycache__" in dirnames:
        shutil.rmtree(os.path.join(dirpath, "__pycache__"))



{%- set counter = count(1) %}

if not silent:

    print(f"""
======================================================================
                        Contratulations!
======================================================================
You just created a new Freva Plugin package.

{% if 'use_reuse' in cookiecutter and cookiecutter.use_reuse == "yes" -%}
+-------------------------------------------------------------+
| IMPORTANT NOTE:                                             |
|                                                             |
|    If you used cruft to create this plugin, please now run  |
|    the following commands to stage the .cruft.json file and |
|    assign it the correct license:                           |
|                                                             |
+-------------------------------------------------------------+

{{ counter | next }}. Set the correct license information:

    $ reuse --root {{ cookiecutter.plugin_slug }} annotate --year {{ cookiecutter.copyright_year }} --license {{ cookiecutter.supplementary_files_license }} --copyright "{{ cookiecutter.copyright_holder }}" {{ cookiecutter.plugin_slug }}/.cruft.json

{{ counter | next }}. Stage the file:

    $ git -C {{ cookiecutter.plugin_slug }} add .cruft.json .cruft.json.license

{%- else %}
+-------------------------------------------------------------+
| IMPORTANT NOTE:                                             |
|                                                             |
|    You can now push everything to gitlab!                   |
|                                                             |
+-------------------------------------------------------------+

{{ counter | next }}. Stage the file .cruft.json:

    $ git -C {{ cookiecutter.plugin_slug }} add .cruft.json

{%- endif %}

{{ counter | next }}. Make your first commit via

    $ git -C {{ cookiecutter.plugin_slug }} commit -m "Initial commit"

{{ counter | next }}. Create the repository at https://{{ cookiecutter.gitlab_host }}/{{ cookiecutter.gitlab_username }}/{{ cookiecutter.plugin_slug }}
   via

    $ git -C {{ cookiecutter.plugin_slug }} push -u origin  {default_branch}

   (You might need to create it through the web-interface first).

+-------------------------------------------------------------+
| IMPORTANT NOTE:                                             |
|                                                             |
|    It's basically provide you python-based linters and      |
|    unit-test tools! To adopt the created sample plugin      |
|    you can adjust Makefile and .github-ci.yml to support    |
|    yout desired programming language.                       |
|                                                             |
+-------------------------------------------------------------+
""")
