"""
{{cookiecutter.plugin_title}} plugin API wrapper.
"""

######################################################
# import commonly used libraries. You can add/remove
# libraries as needed
######################################################

import json
import os
import subprocess
from pathlib import Path
from typing import List, Optional

import freva  # noqa
from evaluation_system.api import plugin
from evaluation_system.api.parameters import (
    Bool,
    Directory,
    Float,
    InputDirectory,
    Integer,
    ParameterDictionary,
    SelectField,
    SolrField,
    String,
)
from evaluation_system.model.user import User


class {{cookiecutter.plugin_folder}}(plugin.PluginAbstract):

    ######################################################
    # Define the details for the plugin
    ######################################################

    # [MANDATORY]: Version of the {{cookiecutter.plugin_title}} plugin (major, minor, patch) * Mandatory
    __version__ = (0, 0, 0)
    # [MANDATORY]: Short Description of the {{cookiecutter.plugin_title}} plugin * Mandatory
    __short_description__: str = "{{cookiecutter.plugin_short_description}}"
    # [OPTIONAL]: category this plugin belongs to
    __category__: Optional[str] = "something"
    # [OPTIONAL]: tags, that are the plugin can be described with and found by
    __tags__: Optional[List[str]] = ["something", "somethingelse"]
    # [OPTIONAL]: author of the plugin
    tool_developer = {
        "name": "{{cookiecutter.plugin_authors}}",
        "email": "{{cookiecutter.plugin_author_emails}}"
    }
    # [OPTIONAL]: long description of the author of the plugin
    __long_description__: Optional[str] = "{{cookiecutter.plugin_title}} plugin long description"


    ######################################################
    # [MANDATORY]: Define the parameters for the plugin.
    # REPLACE THE PARAMETERS WITH YOUR OWN PARAMETERS
    ######################################################

    __parameters__ = ParameterDictionary(
        SolrField(
            name="project",
            facet="project",
            group=1,
            multiple=False,
            predefined_facets=None,
            editable=True,
            default="nukleus",
        ),
        SolrField(
            name="product",
            facet="product",
            group=1,
            multiple=False,
            predefined_facets=None,
            editable=True,
            default="ceu-3",
        ),
        SolrField(
            name="experiment",
            facet="experiment",
            group=1,
            multiple=False,
            predefined_facets=None,
            editable=True,
            default="historical",
        ),
        SolrField(
            name="models",
            facet="models",
            group=1,
            multiple=False,
            predefined_facets=None,
            editable=True,
            default="miroc-miroc6-clmcom-kit-cclm-6-0-clm2-nukleus-x2yn2-v1",
        ),
        SolrField(
            name="time_frequency",
            facet="time_frequency",
            group=1,
            multiple=False,
            predefined_facets=None,
            editable=True,
            default="1hr",
        ),
        SolrField(
            name="variable",
            facet="variable",
            group=1,
            multiple=False,
            predefined_facets=None,
            editable=True,
        ),
        String(
            name="owner",
            default="None",
            help=(
                "Add the name of owner of the data in the metadata."
            ),
        ),
        Bool(
            name="debug",
            default=True,
        ),
        Integer(
            name="multiply_by",
            default=1,
            help="Multiply the output mean values by this interger number.",
        ),
        Float(
            name="substraction_from",
            default=1.0,
            help="Substract multiplied mean by this float value from the output mean values.",
        ),
        InputDirectory(
            name="inputdirectory",
            default=None,
            help="Directory containing input files.",
        ),
        SelectField(
            name="number_of_output",
            options={"1": "one", "2": "two"},
            default="one",
        ),
        Directory(
            name="path_output",
            default=None,
            help="Directory where output will be stored.",
        )
    )

    def sample_freva_usage(self):
        """
        This is a sample function to demonstrate how to use the freva inside of
        your plugin. For example, you can use the freva to get the list of
        data based on the search criteria.

        ATTENTION: Since this sample function is designed to only demonestrate the
         functionality of function in wrapper, you can simply remove or replace it
         with your own pre-process function.
        """

        _list = freva.databrowser(
            project="nukleus",
            product="ceu-3",
            experiment="historical",
            model="miroc-miroc6-clmcom-kit-cclm-6-0-clm2-nukleus-x2yn2-v1",
            time_frequency="1hr",
            variable="pr"
        )

        return _list

    def prepare_html_output(self, path_output):
        """
        Prepare the HTML output for the plugin in
        the result section.
        """
        result = subprocess.run(
            ["ls", "-la", path_output], capture_output=True, text=True
        )
        ls_output = result.stdout
        current_user = User()
        css_styles = """                body {
                    font-family: -apple-system, BlinkMacSystemFont, "Segoe UI", Roboto, sans-serif;
                    margin: 0;
                    padding: 20px;
                    line-height: 1.6;
                    background-color: white;
                    display: flex;
                    flex-direction: column;
                    align-items: center;
                    min-height: 100vh;
                }
                .info-text {
                    font-size: 16px;
                    max-width: 800px;
                    margin: 0 auto 20px auto;
                    color: #0c5460;
                    background-color: #d1ecf1;
                    border: 1px solid #bee5eb;
                    padding: 20px;
                    border-radius: 8px;
                    width: 100%;
                    box-sizing: border-box;
                }
                .terminal {
                    background-color: #2d2d2d;
                    border-radius: 8px;
                    box-shadow: 0 10px 20px rgba(0,0,0,0.3);
                    width: 100%;
                    max-width: 800px;
                    overflow: hidden;
                }
                .terminal-header {
                    background-color: #3d3d3d;
                    padding: 10px;
                    display: flex;
                    align-items: center;
                }
                .terminal-buttons {
                    display: flex;
                    gap: 8px;
                }
                .terminal-button {
                    width: 12px;
                    height: 12px;
                    border-radius: 50%;
                }
                .terminal-button.close {background-color: #ff5f56;}
                .terminal-button.minimize {background-color: #ffbd2e;}
                .terminal-button.maximize {background-color: #27c93f;}
                .terminal-title {
                    color: #ababab;
                    font-size: 14px;
                    margin-left: 20px;
                }
                .terminal-body {
                    padding: 20px;
                    font-family: "Menlo", "Monaco", "Consolas", monospace;
                    font-size: 14px;
                    line-height: 1.5;
                    color: #fff;
                    overflow-x: auto;
                    -webkit-overflow-scrolling: touch;
                }
                .prompt {
                    color: #98c379;
                    margin-bottom: 10px;
                    word-break: break-all;
                }
                .command {
                    color: #61afef;
                    margin-left: 8px;
                }
                .output {
                    color: #abb2bf;
                    white-space: pre-wrap;
                    margin: 10px 0 20px 0;
                }
                @media (max-width: 480px) {
                    body { padding: 10px; }
                    .terminal-body { padding: 15px; }
                }
                """

        html_content = f"""<!DOCTYPE html>
        <html>
        <head>
            <meta charset="UTF-8">
            <meta name="viewport" content="width=device-width, initial-scale=1.0, maximum-scale=1.0">
            <style>
                {css_styles}
            </style>
        </head>
        <body>
            <div class="info-text">
                You can directly download the produced data from the link above or login to levante and and proceed with the exploration of the data.
            </div>
            <div class="terminal">
                <div class="terminal-header">
                    <div class="terminal-buttons">
                        <div class="terminal-button close"></div>
                        <div class="terminal-button minimize"></div>
                        <div class="terminal-button maximize"></div>
                    </div>
                    <span class="terminal-title">Terminal</span>
                </div>
                <div class="terminal-body">
                    <div class="prompt">
                        local:~$ <span class="command">ssh {current_user.getName()}@levante.dkrz.de</span>
                    </div>
                    <div class="prompt">
                        {current_user.getName()}@levante.dkrz.de:~$ <span class="command">cd {path_output}</span>
                    </div>
                    <div class="prompt">
                        {current_user.getName()}@levante.dkrz.de:{path_output}$ <span class="command">ls -la {path_output}</span>
                    </div>
                    <div class="output">{ls_output}</div>
                </div>
            </div>
        </body>
        </html>"""

        html_output_path = os.path.join(path_output, "02_view.html")
        with open(html_output_path, "w") as f:
            f.write(html_content)

    def run_tool(self, config_dict=None):
        """
        Run the {{cookiecutter.plugin_title}} plugin. This function is called by the Freva.
        """
        # [MANDATORY]: Using the follwoing lines you can define the output directory. DON'T CHANGE THIS
        out_dir = self._special_variables.substitute({"d": "$USER_OUTPUT_DIR"})
        out_dir = Path(out_dir["d"]) / str(self.rowid)
        out_dir.mkdir(exist_ok=True, parents=True)
        config_dict["path_output"] = str(out_dir)
        # [OPTIONAL]: Using the follwoing lines you can define the output directory

        user = User()
        scratch = os.getenv("SCRATCH", f"/scratch/{user.getName()[0]}/{user.getName()}")
        cache_dir = Path(f"{scratch}/{{cookiecutter.plugin_folder}}/{self.rowid}")
        cache_dir.mkdir(exist_ok=True, parents=True)
        config_dict["cache_dir"] = str(cache_dir)


        # Define the configuration JSON file. DON'T CHANGE THIS
        json_file = cache_dir / "config.json"

        # Define the error log file. DON'T CHANGE THIS
        error_file = cache_dir / "error.log"

        config_dict["files"] = list(self.sample_freva_usage())
        # [MANDATORY]: Dump the configuration dictionary to the json_file. DON'T CHANGE THIS
        with json_file.open("w") as json_f:
            json.dump(config_dict, json_f, indent=4)

        # [MANDATORY]: Define the path to the environment. Please be aware that this directory
        # will be created when the plugin environment is installed via `make mamba-dep-install`
        # or `make conda-dep-install`. But we recommend to use `make mamba-dep-install` for
        # the faster installation of the plugin environment.
        python_env = Path(__file__).parent / "plugin_env" / "bin" / "python"

        # Define the path to the main tool runner script. Please note that we
        # provided a sample script in the src directory of the plugin to demonstrate
        # how to run the plugin. You can replace the sample script with your own script.
        #
        # REPLACE THE TOOL PATH WITH YOUR OWN TOOL PATH
        tool_path = (
            Path(__file__).parent
            / "src"
            / "{{ cookiecutter.plugin_folder }}"
            / "sample_script.py"
        )
        cmd = [str(python_env), str(tool_path), str(json_file)]

        # [MANDATORY]: Run the tool. Please note that the error log file is used to capture the error messages
        with error_file.open("w") as error_f:
            self.call(cmd, stderr=error_f)

        # [OPTIONAL]: Prepare the HTML output
        self.prepare_html_output(config_dict["path_output"])

        # [OPTIONAL]: Add the output to the data browser
        self.add_output_to_databrowser(
            plugin_output=config_dict["path_output"],
            project=config_dict["project"], # user-<user-id>
            product=config_dict["product"], # f"{config_dict["project"]}.{config_dict["project"]}"
            model=config_dict["models"],
            experiment=config_dict["experiment"],
            time_frequency=config_dict["time_frequency"],
            variable=config_dict["variable"],
        )

        # [MANDATORY]: Return the output directory: DON'T CHANGE THIS
        return self.prepare_output(config_dict["path_output"])
